"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.executePoetryCommand = exports.runPoetryCommand = exports.waitForCommand = exports.runPoetryCommandAsync = exports.CmdStatus = void 0;
const shell = require("shelljs");
var CmdStatus;
(function (CmdStatus) {
    CmdStatus["Done"] = "done";
    CmdStatus["Error"] = "error";
    CmdStatus["ErrorAndDone"] = "error_and_done";
    CmdStatus["Continue"] = "continue";
})(CmdStatus = exports.CmdStatus || (exports.CmdStatus = {}));
function commandSetup(root, args) {
    const cmd = `poetry ${args.join(' ')}`;
    console.info(`Running: ${cmd}`);
    if (!shell.which('poetry')) {
        console.error(`Poetry must be installed to run:\n> ${cmd}`);
        throw new Error();
    }
    if (!shell.pwd().endsWith(root)) {
        const paths = shell.pushd('-q', root);
        if (!paths || !paths.length) {
            return null;
        }
    }
    return cmd;
}
function runPoetryCommandAsync(root, args, checkStatus) {
    return new Promise((resolve, reject) => {
        const cmd = commandSetup(root, args);
        if (!cmd) {
            return reject();
        }
        const child = shell.exec(cmd, { async: true });
        shell.popd('-q');
        let error = false;
        const onComplete = (e) => {
            if (e) {
                console.error('Poetry command failed, exiting');
                child.kill('SIGHUP');
                reject();
            }
            else {
                resolve(child);
            }
        };
        const checkOutput = (data) => {
            const status = checkStatus(data);
            switch (status) {
                case CmdStatus.Continue:
                    break;
                case CmdStatus.Error:
                    error = true;
                    break;
                case CmdStatus.ErrorAndDone:
                    onComplete(true);
                    break;
                case CmdStatus.Done:
                    onComplete(error);
                    break;
            }
        };
        child.stderr.on('data', checkOutput);
        child.stdout.on('data', checkOutput);
    });
}
exports.runPoetryCommandAsync = runPoetryCommandAsync;
function waitForCommand(cmd, baseUrl) {
    return new Promise((res) => {
        cmd.on('exit', (code) => {
            res({
                success: code === 0,
                baseUrl,
            });
        });
    });
}
exports.waitForCommand = waitForCommand;
function runPoetryCommand(root, ...args) {
    return () => {
        const cmd = commandSetup(root, args);
        if (!cmd || shell.exec(cmd).code !== 0) {
            shell.popd();
            console.error(`Poetry command failed:\n> ${cmd}`);
            throw new Error();
        }
        shell.popd();
    };
}
exports.runPoetryCommand = runPoetryCommand;
// Run a Poetry command immediately. Useful for executors
function executePoetryCommand(root, ...args) {
    const cmd = commandSetup(root, args);
    if (!cmd || shell.exec(cmd).code !== 0) {
        shell.popd();
        console.error(`Poetry command failed:\n> ${cmd}`);
        return false;
    }
    shell.popd();
    return true;
}
exports.executePoetryCommand = executePoetryCommand;
//# sourceMappingURL=poetry.js.map